@@ -5,16 +5,20 @@ use ckb_db::{
     DBVector, RocksDBTransaction, RocksDBTransactionSnapshot,
 };
 use ckb_db_schema::{
-    Col, COLUMN_BLOCK_BODY, COLUMN_BLOCK_EPOCH, COLUMN_BLOCK_EXT, COLUMN_BLOCK_HEADER,
-    COLUMN_BLOCK_PROPOSAL_IDS, COLUMN_BLOCK_UNCLE, COLUMN_CELL, COLUMN_CELL_DATA,
-    COLUMN_CELL_DATA_HASH, COLUMN_EPOCH, COLUMN_INDEX, COLUMN_META, COLUMN_NUMBER_HASH,
-    COLUMN_TRANSACTION_INFO, COLUMN_UNCLES, META_CURRENT_EPOCH_KEY, META_TIP_HEADER_KEY,
+    Col, COLUMN_BLOCK_BODY, COLUMN_BLOCK_EPOCH, COLUMN_BLOCK_EXT, COLUMN_BLOCK_EXTENSION,
+    COLUMN_BLOCK_HEADER, COLUMN_BLOCK_PROPOSAL_IDS, COLUMN_BLOCK_UNCLE, COLUMN_CELL,
+    COLUMN_CELL_DATA, COLUMN_CELL_DATA_HASH, COLUMN_EPOCH, COLUMN_INDEX, COLUMN_META,
+    COLUMN_NUMBER_HASH, COLUMN_TRANSACTION_INFO, COLUMN_UNCLES, META_CURRENT_EPOCH_KEY,
+    META_TIP_HEADER_KEY,
 };
 use ckb_error::Error;
 use ckb_freezer::Freezer;
 use ckb_types::{
-    core::{BlockExt, BlockView, EpochExt, HeaderView},
-    packed,
+    core::{
+        cell::{CellChecker, CellProvider, CellStatus},
+        BlockExt, BlockView, EpochExt, HeaderView,
+    },
+    packed::{self, OutPoint},
     prelude::*,
 };
 use std::sync::Arc;
@@ -48,6 +52,33 @@ impl<'a> ChainStore<'a> for StoreTransaction {
     }
 }
 
+impl CellProvider for StoreTransaction {
+    fn cell(&self, out_point: &OutPoint, eager_load: bool) -> CellStatus {
+        match self.get_cell(out_point) {
+            Some(mut cell_meta) => {
+                if eager_load {
+                    if let Some((data, data_hash)) = self.get_cell_data(out_point) {
+                        cell_meta.mem_cell_data = Some(data);
+                        cell_meta.mem_cell_data_hash = Some(data_hash);
+                    }
+                }
+                CellStatus::live_cell(cell_meta)
+            }
+            None => CellStatus::Unknown,
+        }
+    }
+}
+
+impl CellChecker for StoreTransaction {
+    fn is_live(&self, out_point: &OutPoint) -> Option<bool> {
+        if self.have_cell(out_point) {
+            Some(true)
+        } else {
+            None
+        }
+    }
+}
+
 pub struct StoreTransactionSnapshot<'a> {
     pub(crate) inner: RocksDBTransactionSnapshot<'a>,
     pub(crate) freezer: Option<Freezer>,
@@ -126,6 +157,13 @@ impl StoreTransaction {
         let txs_len: packed::Uint32 = (block.transactions().len() as u32).pack();
         self.insert_raw(COLUMN_BLOCK_HEADER, hash.as_slice(), header.as_slice())?;
         self.insert_raw(COLUMN_BLOCK_UNCLE, hash.as_slice(), uncles.as_slice())?;
+        if let Some(extension) = block.extension() {
+            self.insert_raw(
+                COLUMN_BLOCK_EXTENSION,
+                hash.as_slice(),
+                &extension.as_slice(),
+            )?;
+        }
         self.insert_raw(
             COLUMN_NUMBER_HASH,
             packed::NumberHash::new_builder()
@@ -157,6 +195,7 @@ impl StoreTransaction {
         let txs_len = block.transactions().len();
         self.delete(COLUMN_BLOCK_HEADER, hash.as_slice())?;
         self.delete(COLUMN_BLOCK_UNCLE, hash.as_slice())?;
+        self.delete(COLUMN_BLOCK_EXTENSION, hash.as_slice())?;
         self.delete(COLUMN_BLOCK_PROPOSAL_IDS, hash.as_slice())?;
         self.delete(
             COLUMN_NUMBER_HASH,
